        # if finalstate<6:
        #     await DORMANT(ws)
        # elif finalstate<15:
        #     await ALERT(ws)
        # elif finalstate<20:
        #     await ACTIVE(ws)
        # else:
        #     print("working")